@@ -18,7 +18,8 @@
 # include <unistd.h>
 #endif  // HAVE_UNISTD_H
 
-#include "common/bit_cursor.h"
+#include "common/bit_reader.h"
+#include "common/bit_writer.h"
 #include "common/dts.h"
 #include "common/endian.h"
 #include "common/list_utils.h"
@@ -146,7 +147,6 @@ find_header(unsigned char const *buf,
   try {
     return find_header_internal(buf, size, header, allow_no_exss_search);
   } catch (...) {
-    mxwarn(Y("DTS_Header problem: not enough data to read header\n"));
     return -1;
   }
 }
@@ -367,6 +367,14 @@ header_t::get_total_num_audio_channels()
   return get_core_num_audio_channels();
 }
 
+unsigned int
+header_t::get_effective_sampling_frequency()
+  const {
+  if (extension_sampling_frequency && *extension_sampling_frequency)
+    return *extension_sampling_frequency;
+  return core_sampling_frequency;
+}
+
 codec_c::specialization_e
 header_t::get_codec_specialization()
   const {
@@ -417,7 +425,7 @@ header_t::decode_core_header(unsigned char const *buf,
                              size_t size,
                              bool allow_no_exss_search) {
   try {
-    auto bc = bit_reader_c{buf, size};
+    auto bc = mtx::bits::reader_c{buf, size};
     bc.skip_bits(32);             // sync word
 
     frametype             = bc.get_bit() ? frametype_e::normal : frametype_e::termination;
@@ -426,10 +434,8 @@ header_t::decode_core_header(unsigned char const *buf,
     num_pcm_sample_blocks = bc.get_bits(7) + 1;
     frame_byte_size       = bc.get_bits(14) + 1;
 
-    if (96 > frame_byte_size) {
-      mxwarn(Y("DTS_Header problem: invalid frame bytes size\n"));
+    if (96 > frame_byte_size)
       return false;
-    }
 
     auto t = bc.get_bits(6);
     if (16 <= t) {
@@ -454,7 +460,7 @@ header_t::decode_core_header(unsigned char const *buf,
     predictor_history_flag     = bc.get_bit();
 
     if (crc_present)
-       bc.skip_bits(16);
+      crc = bc.get_bits(16);
 
     multirate_interpolator     = static_cast<multirate_interpolator_e>(bc.get_bit());
     encoder_software_revision  = bc.get_bits(4);
@@ -492,20 +498,19 @@ header_t::decode_core_header(unsigned char const *buf,
         break;
 
       default:
-        mxwarn(Y("DTS_Header problem: invalid source PCM resolution\n"));
         return false;
     }
 
-    front_sum_difference      = bc.get_bit();
-    surround_sum_difference   = bc.get_bit();
-    t                         = bc.get_bits(4);
-    dialog_normalization_gain = 7 == encoder_software_revision ? -t
-                              : 6 == encoder_software_revision ? -16 - t
-                              :                                  0;
-
-    has_core                  = true;
+    front_sum_difference                   = bc.get_bit();
+    surround_sum_difference                = bc.get_bit();
+    dialog_normalization_gain_bit_position = bc.get_bit_position();
+    t                                      = bc.get_bits(4);
+    dialog_normalization_gain              = 7 == encoder_software_revision ? -t
+                                           : 6 == encoder_software_revision ? -16 - t
+                                           :                                  0;
 
     // Detect DTS HD master audio / high resolution part
+    has_core         = true;
     has_exss         = false;
     exss_part_size   = 0;
     dts_type         = dts_type_e::normal;
@@ -536,7 +541,7 @@ header_t::decode_core_header(unsigned char const *buf,
 }
 
 void
-header_t::parse_lbr_parameters(bit_reader_c &bc,
+header_t::parse_lbr_parameters(mtx::bits::reader_c &bc,
                                substream_asset_t &asset) {
   asset.lbr_size         = bc.get_bits(14) + 1; // size of LBR component in extension substream
   asset.lbr_sync_present = bc.get_bit();
@@ -545,7 +550,7 @@ header_t::parse_lbr_parameters(bit_reader_c &bc,
 }
 
 void
-header_t::parse_xll_parameters(bit_reader_c &bc,
+header_t::parse_xll_parameters(mtx::bits::reader_c &bc,
                                substream_asset_t &asset) {
   asset.xll_size         = bc.get_bits(substream_size_bits) + 1; // size of XLL data in extension substream
   asset.xll_sync_present = bc.get_bit();                         // XLL sync word presence flag
@@ -563,7 +568,7 @@ header_t::parse_xll_parameters(bit_reader_c &bc,
 }
 
 bool
-header_t::decode_lbr_header(bit_reader_c &bc,
+header_t::decode_lbr_header(mtx::bits::reader_c &bc,
                             substream_asset_t &asset) {
   static const unsigned int s_lbr_sampling_frequencies[16] = {
     8000, 16000, 32000, 0,     0,     22050, 44100, 0,
@@ -585,14 +590,18 @@ header_t::decode_lbr_header(bit_reader_c &bc,
 }
 
 bool
-header_t::decode_xll_header(bit_reader_c &bc,
+header_t::decode_xll_header(mtx::bits::reader_c &bc,
                             substream_asset_t &asset) {
   bc.set_bit_position((asset.xll_offset + asset.xll_sync_offset) * 8);
   if (asset.xll_sync_present && (bc.get_bits(32) != static_cast<uint32_t>(sync_word_e::xll)))
     return false;
 
-  if (!has_core && !(asset.extension_mask & exss_lbr))
-    core_sampling_frequency = substream_assets[0].max_sample_rate;
+  if (!(asset.extension_mask & exss_lbr)) {
+    if (!has_core)
+      core_sampling_frequency = substream_assets[0].max_sample_rate;
+    else
+      extension_sampling_frequency.reset(substream_assets[0].max_sample_rate);
+  }
 
   return true;
 
@@ -626,7 +635,7 @@ header_t::decode_xll_header(bit_reader_c &bc,
 }
 
 bool
-header_t::decode_asset(bit_reader_c &bc,
+header_t::decode_asset(mtx::bits::reader_c &bc,
                        substream_asset_t &asset) {
   auto descriptor_pos  = bc.get_bit_position();
   auto descriptor_size = bc.get_bits(9) + 1;
@@ -797,7 +806,7 @@ bool
 header_t::decode_x96_header(unsigned char const *buf,
                              size_t size) {
   try {
-    auto bc = bit_reader_c{buf, size};
+    auto bc = mtx::bits::reader_c{buf, size};
     bc.skip_bits(32);             // sync word
 
     auto x96_size = bc.peek_bits(12) + 1;
@@ -815,7 +824,7 @@ bool
 header_t::decode_exss_header(unsigned char const *buf,
                              size_t size) {
   try {
-    auto bc = bit_reader_c{buf, size};
+    auto bc = mtx::bits::reader_c{buf, size};
     bc.skip_bits(32);             // sync word
     bc.skip_bits(8);              // user defined
     auto substream_index  = bc.get_bits(2);
@@ -840,8 +849,8 @@ header_t::decode_exss_header(unsigned char const *buf,
     if (static_fields_present) {
       reference_clock_code     = bc.get_bits(2);             // reference clock code
       substream_frame_duration = (bc.get_bits(3) + 1) * 512; // substream frame duration
-      if (bc.get_bit())                                      // timecode presence flag
-        bc.skip_bits(32 + 4);                                // timecode data
+      if (bc.get_bit())                                      // timestamp presence flag
+        bc.skip_bits(32 + 4);                                // timestamp data
 
       num_presentations = bc.get_bits(3) + 1;
       num_assets        = bc.get_bits(3) + 1;
@@ -895,37 +904,32 @@ convert_14_to_16_bits(const unsigned short *src,
   // srcwords has to be a multiple of 8!
   // you will get (srcbytes >> 3)*7 destination words!
 
-  const unsigned long l = srcwords >> 3;
+  int const l = srcwords >> 3;
 
-  for (unsigned long b = 0; b < l; b++) {
+  for (int b = 0; b < l; b++) {
     unsigned short src_0 = (src[0] >>  8) | (src[0] << 8);
     unsigned short src_1 = (src[1] >>  8) | (src[1] << 8);
-    // 14 + 2
+    unsigned short src_2 = (src[2] >>  8) | (src[2] << 8);
+    unsigned short src_3 = (src[3] >>  8) | (src[3] << 8);
+    unsigned short src_4 = (src[4] >>  8) | (src[4] << 8);
+    unsigned short src_5 = (src[5] >>  8) | (src[5] << 8);
+    unsigned short src_6 = (src[6] >>  8) | (src[6] << 8);
+    unsigned short src_7 = (src[7] >>  8) | (src[7] << 8);
+
     unsigned short dst_0 = (src_0  <<  2) | ((src_1 & 0x3fff) >> 12);
-    dst[0]               = (dst_0  >>  8) | (dst_0            <<  8);
-    // 12 + 4
-    unsigned short src_2 = (src[2] >>  8) | (src[2]           <<  8);
     unsigned short dst_1 = (src_1  <<  4) | ((src_2 & 0x3fff) >> 10);
-    dst[1]               = (dst_1  >>  8) | (dst_1            <<  8);
-    // 10 + 6
-    unsigned short src_3 = (src[3] >>  8) | (src[3]           <<  8);
     unsigned short dst_2 = (src_2  <<  6) | ((src_3 & 0x3fff) >>  8);
-    dst[2]               = (dst_2  >>  8) | (dst_2            <<  8);
-    // 8  + 8
-    unsigned short src_4 = (src[4] >>  8) | (src[4]           <<  8);
     unsigned short dst_3 = (src_3  <<  8) | ((src_4 & 0x3fff) >>  6);
-    dst[3]               = (dst_3  >>  8) | (dst_3            <<  8);
-    // 6  + 10
-    unsigned short src_5 = (src[5] >>  8) | (src[5]           <<  8);
     unsigned short dst_4 = (src_4  << 10) | ((src_5 & 0x3fff) >>  4);
-    dst[4]               = (dst_4  >>  8) | (dst_4            <<  8);
-    // 4  + 12
-    unsigned short src_6 = (src[6] >>  8) | (src[6]           <<  8);
     unsigned short dst_5 = (src_5  << 12) | ((src_6 & 0x3fff) >>  2);
+    unsigned short dst_6 = (src_6  << 14) |  (src_7 & 0x3fff);
+
+    dst[0]               = (dst_0  >>  8) | (dst_0            <<  8);
+    dst[1]               = (dst_1  >>  8) | (dst_1            <<  8);
+    dst[2]               = (dst_2  >>  8) | (dst_2            <<  8);
+    dst[3]               = (dst_3  >>  8) | (dst_3            <<  8);
+    dst[4]               = (dst_4  >>  8) | (dst_4            <<  8);
     dst[5]               = (dst_5  >>  8) | (dst_5            <<  8);
-    // 2  + 14
-    unsigned short src_7 = (src[7] >>  8) | (src[7]           <<  8);
-    unsigned short dst_6 = (src_6  << 14) | (src_7 & 0x3fff);
     dst[6]               = (dst_6  >>  8) | (dst_6            <<  8);
 
     dst                 += 7;
@@ -979,6 +983,42 @@ detect(const void *src_buf,
   return is_dts;
 }
 
+void
+remove_dialog_normalization_gain(unsigned char *buf,
+                                 std::size_t size) {
+  static debugging_option_c s_debug{"dts_remove_dialog_normalization_gain|remove_dialog_normalization_gain"};
+
+  header_t header;
+
+  if (!header.decode_core_header(buf, size, true))
+    return;
+
+  if (!header.has_core || !header.dialog_normalization_gain_bit_position || (size < header.frame_byte_size))
+    return;
+
+  unsigned int const removed_level = 0;
+
+  if (header.dialog_normalization_gain == removed_level) {
+    mxdebug_if(s_debug,
+               boost::format("no need to remove the dialog normalization, it's already set to %1% (%2% dB); CRC: %3%\n")
+               % removed_level % header.dialog_normalization_gain % (header.crc ? (boost::format("%|1$04x|") % *header.crc).str() : "—"s));
+    return;
+  }
+
+  mtx::bits::reader_c r{buf, size};
+  mtx::bits::writer_c w{buf, size};
+
+  r.set_bit_position(header.dialog_normalization_gain_bit_position);
+  w.set_bit_position(header.dialog_normalization_gain_bit_position);
+
+  auto current_level = r.get_bits(4);
+  w.put_bits(4, removed_level);
+
+  mxdebug_if(s_debug,
+             boost::format("changing dialog normalization from %1% (%2% dB) to %3%; CRC: %4%\n")
+             % current_level % header.dialog_normalization_gain % removed_level % (header.crc ? (boost::format("%|1$04x|") % *header.crc).str() : "—"s));
+}
+
 bool
 operator ==(const header_t &h1,
             const header_t &h2) {
