@@ -1418,8 +1418,6 @@ ogm_v_theora_demuxer_c::ogm_v_theora_demuxer_c(ogm_reader_c *p_reader)
   : ogm_demuxer_c(p_reader)
 {
   codec = codec_c::look_up(codec_c::type_e::V_THEORA);
-
-  memset(&theora, 0, sizeof(theora_identification_header_t));
 }
 
 void
@@ -1535,8 +1533,7 @@ ogm_v_vp8_demuxer_c::create_packetizer() {
 
   ptzr_obj->set_video_pixel_width(pixel_width);
   ptzr_obj->set_video_pixel_height(pixel_height);
-  ptzr_obj->set_video_display_width(display_width);
-  ptzr_obj->set_video_display_height(display_height);
+  ptzr_obj->set_video_display_dimensions(display_width, display_height, generic_packetizer_c::ddu_pixels, OPTION_SOURCE_BITSTREAM);
   ptzr_obj->set_track_default_duration(default_duration);
 
   show_packetizer_info(m_ti.m_id, ptzr_obj);
@@ -1614,8 +1611,6 @@ ogm_s_kate_demuxer_c::ogm_s_kate_demuxer_c(ogm_reader_c *p_reader)
   : ogm_demuxer_c(p_reader)
 {
   codec = codec_c::look_up(codec_c::type_e::S_KATE);
-
-  memset(&kate, 0, sizeof(kate_identification_header_t));
 }
 
 void
