@@ -1,15 +1,16 @@
-#ifndef MTX_MKVTOOLNIX_GUI_UTIL_STRING_H
-#define MTX_MKVTOOLNIX_GUI_UTIL_STRING_H
+#pragma once
 
 #include "common/common_pch.h"
 
+#include <QRegularExpression>
+
 class QString;
 class QStringList;
 
 namespace mtx { namespace gui { namespace Util {
 
 enum EscapeMode {
-  EscapeMkvtoolnix,
+  EscapeJSON,
   EscapeShellUnix,
   EscapeShellCmdExeArgument,
   EscapeShellCmdExeProgram,
@@ -34,6 +35,21 @@ QString displayableDate(QDateTime const &date);
 
 QString itemFlagsToString(Qt::ItemFlags const &flags);
 
-}}}
+QString mapToTopLevelCountryCode(QString const &countryCode);
 
-#endif  // MTX_MKVTOOLNIX_GUI_UTIL_STRING_H
+QString replaceApplicationDirectoryWithMtxVariable(QString string);
+QString replaceMtxVariableWithApplicationDirectory(QString string);
+
+class DeferredRegularExpression {
+private:
+  std::unique_ptr<QRegularExpression> m_re;
+  QString m_pattern;
+  QRegularExpression::PatternOptions m_options;
+
+public:
+  DeferredRegularExpression(QString const &pattern, QRegularExpression::PatternOptions options = QRegularExpression::NoPatternOption);
+  ~DeferredRegularExpression();
+  QRegularExpression &operator *();
+};
+
+}}}
