@@ -76,7 +76,7 @@ class SimpleTest
   end
 
   def clean_tmp
-    File.unlink(@tmp) if File.exists?(@tmp) && (ENV["KEEP_TMPFILES"] != "1")
+    File.unlink(@tmp) if @tmp && File.exists?(@tmp) && !ENV["KEEP_TMPFILES"].nil? && (ENV["KEEP_TMPFILES"] != "1")
     @tmp = nil
   end
 
@@ -105,16 +105,26 @@ class SimpleTest
     @blocks[:tests] << { :name => name, :block => block }
   end
 
+  def exit_code_string exit_code
+    return exit_code == 0 ? 'ok'      \
+         : exit_code == 1 ? 'warning' \
+         : exit_code == 2 ? 'error'   \
+         :                  "unknown(#{exit_code})"
+  end
+
   def test_merge file, *args
-    options             = args.extract_options!
-    full_command_line   = [ options[:args], file ].flatten.join(' ')
-    options[:name]    ||= full_command_line
+    options                 = args.extract_options!
+    full_command_line       = [ options[:args], file ].flatten.join(' ')
+    options[:name]        ||= full_command_line
+    options[:result_type] ||= :hash
     @blocks[:tests] << {
       :name  => full_command_line,
       :block => lambda {
-        output = options[:output] || tmp
-        merge full_command_line, :exit_code => options[:exit_code], :output => output
-        result = options[:exit_code] == :error ? 'error' : hash_file(output)
+        output       = options[:output] || tmp
+        _, exit_code = *merge(full_command_line, :exit_code => options[:exit_code], :output => output)
+        result       = options[:exit_code]   == :error ? 'error'           \
+                     : options[:result_type] == :hash  ? hash_file(output) \
+                     :                                   exit_code_string(exit_code)
 
         clean_tmp unless options[:keep_tmp]
 
@@ -125,13 +135,17 @@ class SimpleTest
 
   def test_identify file, *args
     options             = args.extract_options!
-    options[:verbose]   = true if options[:verbose].nil?
-    full_command_line   = [ options[:verbose] ? "--identify-verbose" : "--identify", options[:args], file ].flatten.join(' ')
+    options[:format]    = :json if options[:format].nil?
+    full_command_line   = [ "--identification-format", options[:format].to_s, "--identify", options[:args], file ].flatten.join(' ')
     options[:name]    ||= full_command_line
     @blocks[:tests] << {
       :name  => full_command_line,
       :block => lambda {
-        sys "../src/mkvmerge #{full_command_line} > #{tmp}", :exit_code => options[:exit_code]
+        sys "../src/mkvmerge #{full_command_line} --engage no_variable_data > #{tmp}", :exit_code => options[:exit_code]
+
+        text = IO.readlines(tmp).reject { |line| %r{^\s*"identification_format_version":\s*\d+}.match(line) }.join('')
+        File.open(tmp, 'w') { |tmp_file| tmp_file.puts text }
+
         if options[:filter]
           text = options[:filter].call(IO.readlines(tmp).join(''))
           File.open(tmp, 'w') { |tmp_file| tmp_file.puts text }
@@ -162,8 +176,8 @@ class SimpleTest
     @blocks[:tests] << {
       :name  => full_command_line,
       :block => lambda {
-        sys "../src/mkvmerge --identify-verbose #{full_command_line} > #{tmp}", :exit_code => 3
-        %r{unsupported container}.match(IO.readlines(tmp).first || '') ? :ok : :bad
+        json = identify_json full_command_line, :exit_code => 3
+        (json["container"]["recognized"] == true) && (json["container"]["supported"] == false) ? :ok : :bad
       },
     }
   end
@@ -204,6 +218,7 @@ class SimpleTest
         result = test[:block].call
       rescue RuntimeError => ex
         show_message "Test case '#{self.class.name}', sub-test '#{test[:name]}': #{ex}"
+        result = :failed
       end
       result
     end
@@ -221,19 +236,17 @@ class SimpleTest
 
     output  = options[:output] || self.tmp
     command = "../src/mkvmerge --engage no_variable_data -o #{output} #{args.first}"
-    self.sys command, :exit_code => options[:exit_code]
+    self.sys command, :exit_code => options[:exit_code], :no_result => options[:no_result]
   end
 
   def identify *args
     options = args.extract_options!
     fail ArgumentError if args.empty?
 
-    verbose = !options[:verbose].nil? ? options[:verbose]                             : true
-    format  = options[:format]        ? options[:format].to_s.downcase.gsub(/_/, '-') : verbose ? 'verbose-text' : 'text'
-
+    format  = options[:format] ? options[:format].to_s.downcase.gsub(/_/, '-') : 'text'
     command = "../src/mkvmerge --identify --identification-format #{format} --engage no_variable_data #{args.first}"
 
-    self.sys command, :exit_code => options[:exit_code]
+    self.sys command, :exit_code => options[:exit_code], :no_result => options[:no_result]
   end
 
   def identify_json *args
@@ -242,7 +255,7 @@ class SimpleTest
 
     command = "../src/mkvmerge --identify --identification-format json --engage no_variable_data #{args.first}"
 
-    output, _ = self.sys(command, :exit_code => options[:exit_code])
+    output, _ = self.sys(command, :exit_code => options[:exit_code], :no_result => options[:no_result])
 
     return JSON.load(output.join(''))
   end
@@ -255,7 +268,7 @@ class SimpleTest
     output  = "> #{output}" unless %r{^[>\|]}.match(output)
     output  = '' if options[:output] == :return
     command = "../src/mkvinfo --engage no_variable_data --ui-language en_US #{args.first} #{output}"
-    self.sys command, :exit_code => options[:exit_code]
+    self.sys command, :exit_code => options[:exit_code], :no_result => options[:no_result], :dont_record_command => options[:dont_record_command]
   end
 
   def extract *args
@@ -265,7 +278,7 @@ class SimpleTest
     mode     = options[:mode] || :tracks
     command  = "../src/mkvextract --engage no_variable_data #{mode} #{args.first} " + options.keys.select { |key| key.is_a?(Numeric) }.sort.collect { |key| "#{key}:#{options[key]}" }.join(' ')
 
-    self.sys command, :exit_code => options[:exit_code]
+    self.sys command, :exit_code => options[:exit_code], :no_result => options[:no_result]
   end
 
   def propedit file_name, *args
@@ -273,9 +286,9 @@ class SimpleTest
     fail ArgumentError if args.empty?
 
     command = "../src/mkvpropedit --engage no_variable_data #{file_name} #{args.first}"
-    *result = self.sys command, :exit_code => options[:exit_code]
+    *result = self.sys command, :exit_code => options[:exit_code], :no_result => options[:no_result]
 
-    self.sys "../src/tools/ebml_validator -M #{file_name}" if FileTest.exists?("../src/tools/ebml_validator")
+    self.sys "../src/tools/ebml_validator -M #{file_name}", dont_record_command: true if FileTest.exists?("../src/tools/ebml_validator")
 
     return *result
   end
@@ -286,7 +299,7 @@ class SimpleTest
     fail ArgumentError if args.empty?
 
     command    = args.shift
-    @commands << command
+    @commands << { :command => command, :no_result => options[:no_result] } unless options[:dont_record_command]
 
     if !%r{>}.match command
       temp_file = Tempfile.new('mkvtoolnix-test-output')
@@ -306,6 +319,13 @@ class SimpleTest
     return exit_code
   end
 
+  def cp source, target, *args
+    options = args.extract_options!
+    options[:no_result] = true unless options.key?(:no_result)
+
+    sys "cp '#{source}' '#{target}'", options
+  end
+
   def error reason
     show_message "  Failed. Reason: #{reason}"
     raise "test failed"
@@ -314,16 +334,38 @@ class SimpleTest
   def json_schema_identification
     return @@json_schema_identification if @@json_schema_identification
 
-    require "json_schema"
-
     json_store = JsonSchema::DocumentStore.new
     parser     = JsonSchema::Parser.new
     expander   = JsonSchema::ReferenceExpander.new
-    schema     = parser.parse JSON.load(File.read("../doc/json-schema/mkvmerge-identification-output-schema-v3.json"))
+    version    = IO.readlines("../src/merge/id_result.h").detect { |line| /#define +ID_JSON_FORMAT_VERSION\b/.match line }.gsub(/.* .* /, '').chop
+    schema     = parser.parse JSON.load(File.read("../doc/json-schema/mkvmerge-identification-output-schema-v#{version}.json"))
 
     expander.expand(schema, store: json_store)
     json_store.add_schema schema
 
     @@json_schema_identification = schema
   end
+
+  def json_summarize_tracks json
+    basics     = %w{id type codec}
+    properties = %w{language number audio_channels audio_sampling_frequency pixel_dimensions}
+
+    json["tracks"].
+      map do |t|
+        (basics.map { |b| t[b] } +
+         properties.map { |p| t["properties"][p] }).
+        reject(&:nil?).
+        join("+").
+        gsub(%r{[:/+-]+}, "_")
+      end.
+      join("/")
+  end
+
+  def get_doc_type_versions file_name
+    output            = info(file_name, :output => :return, :dont_record_command => true).join('')
+    type_version      = /Document type version:\s+(\d+)/.match(output)      ? $1 : nil
+    type_read_version = /Document type read version:\s+(\d+)/.match(output) ? $1 : nil
+
+    return type_version, type_read_version
+  end
 end
